@@ -1,6 +1,6 @@
 /*
  * GProstats - GProFTPD Statistics generator.
- * Copyright 2003, 2004, 2005 Magnus-swe <magnus-swe@telia.com>
+ * Copyright 2003 - 2006 Magnus Loef (Magnus-swe) <magnus-swe@telia.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,8 +33,7 @@
 #include "../config.h"
 #include <stdio.h>
 #include <stdlib.h>
-
-#include "functions.h"
+#include "chars_are_digits.h"
 
 /* Solaris issue */
 #ifdef HAVE_STDINT_H
@@ -70,10 +69,11 @@ show_usage(char conf[8192], char xferlog[8192])
     printf("-c      %s\n", conf);
     printf("-x      %s\n", xferlog);
     printf("-w      Make welcome messages for all ftpusers with this name.\n");
-    printf("-html   Also write html output [/path/to/x.html]\n\n");
+    printf("-html   Make a html ftp statistics page in the specified location.\n\tIE: [/var/www/html/ftp_stats.html]\n\n");
     printf("The default paths are [/etc/proftpd.conf] and [/var/log/xferlog]\n");
     printf("The default welcome for gproftpd is [welcome.msg]\n\n");
-    printf("If neither of the -w or -html flags are used it will just print the results to the screen.\n\n");
+    printf("If neither of the -w or -html flags are used it will just print the results to the screen.\n");
+    printf("Only uploads and downloads above 1023 Bytes are counted.\n\n");
 }
 
 
@@ -95,8 +95,6 @@ main(int argc, char *argv[])
     /* The name of the server */
     char server_name[8192]="";
 
-    /* Hello sudo masters ;) */
-    
     /* HTML Part 1 (the lengths are max input from the entries) */
     /* Can contain: static HTML(16384), username(128)*20 (2560), ul/dl KB(50)*20 (1000), 
      * server name (1000), datestamp (100) and html_welcome1 */
@@ -107,11 +105,11 @@ main(int argc, char *argv[])
     char html_welcome1[5000]="";
 
     int row=0, old_row=0, max_rows=0, count=0, x=0, z=0, top_ul=0, top_dl=0, timez=0, timex=0, once=0;
-    int found_virtualhost=0;
+    int found_virtualhost = 0;
     
     /* Default settings, specified by configure and installed by a superuser */
-    strcpy(conf, GP_CONF);
-    strcpy(xferlog, GP_XFERLOG);
+    strcpy(conf, PROFTPD_CONF);
+    strcpy(xferlog, XFER_LOG);
 
     /* Check length of arguments */
     for(x=0; argv[x]!=NULL; x++)
@@ -142,6 +140,8 @@ main(int argc, char *argv[])
 	}
     }
 
+    printf("\n");
+
     /* Check for user-supplied args */
     for(x=0; x<argc; x++)
     {
@@ -171,11 +171,14 @@ main(int argc, char *argv[])
 	if( argc>x+1 )
 	if( strstr(argv[x], "-html") && strstr(argv[x+1], "/") )
 	{
-	    printf("Making html statistics: %s\n", argv[x+1]);
+	    printf("Making a html statistics page here: %s\n", argv[x+1]);
 	    strcpy(html, argv[x+1]);
 	}
     }
 
+    printf("\n");
+
+
     /* Get usernames and homedirectories */
     if((fp = fopen(conf, "r"))==NULL)
     {
@@ -189,13 +192,17 @@ main(int argc, char *argv[])
 
     old_buffer = allocate(size_buffer);
 
+    if( size_buffer > 1 )
     while(fgets(old_buffer, size_buffer, fp)!=NULL)
     {
+	if( strlen(old_buffer) > 8000 )
+	  continue;
+
 	if( strstr(old_buffer, "<Anonymous /") )
 	max_rows++;
 
 	if( strstr(old_buffer, "<VirtualHost") )
-	  found_virtualhost=1;
+	  found_virtualhost = 1;
 	
 	/* Get the servername for the default server */
 	if( ! found_virtualhost )
@@ -220,12 +227,16 @@ main(int argc, char *argv[])
     dir = allocate(8192);
     rewind(fp);
 
+    if( size_buffer > 1 )
     while(fgets(old_buffer, size_buffer, fp)!=NULL)
     {
+	if( strlen(old_buffer) > 8000 )
+	  continue;
+
 	if( strstr(old_buffer, "<Anonymous /") )
 	{
 	    /* Homedirectories */
-	    snprintf(dir, *old_buffer, old_buffer+11); /* 11 = "<Anonymous " */
+	    snprintf(dir, 8000, "%s", old_buffer+11);
 	    for(x=0; dir[x]; x++)
 	    {
 		if(dir[x]=='>')
@@ -234,31 +245,37 @@ main(int argc, char *argv[])
 		    break;
 		}
 	    }	    
-	    strcpy(userlist[row].dir, dir);
+	    snprintf(userlist[row].dir, 8000, "%s", dir);
+
 	    while(fgets(old_buffer, size_buffer, fp)!=NULL)
 	    {
+		if( strlen(old_buffer) > 8000 )
+		  continue;
+
 		if( strstr(old_buffer, "</Anonymous>") )
-		{
-		    break;
-		}    
+		  break;
+    
 		/* Username */
 		if( strstr(old_buffer, "User ") && ! strstr(old_buffer, "DirFakeUser") 
 		&& ! strstr(old_buffer, "AllowUser") && ! strstr(old_buffer, "DenyUser") ) 
 		{
 		    sscanf(old_buffer, "%*s %s", user);
-		    strcpy(userlist[row].user, user);
+		    snprintf(userlist[row].user, 8000, "%s", user);
+
 		    /* Init all to be updated */
-		    userlist[row].update=1;
+		    userlist[row].update = 1;
 
 		    /* Locked .. dont update the welcome message  */
 		    while(fgets(old_buffer, size_buffer, fp)!=NULL)
 		    {
+			if( strlen(old_buffer) > 8000 )
+			  continue;
+
 			if( strstr(old_buffer, "#gplockstats") )
 			{
-			    userlist[row].update=0;
+			    userlist[row].update = 0;
 			    row--;
 			} 
-
 			if( strstr(old_buffer, "</Anonymous>") )
 			  break;
 		    }
@@ -287,11 +304,17 @@ main(int argc, char *argv[])
     fseek(fp, 0, SEEK_END);
     size_buffer = ftell(fp);
     rewind(fp);
+    
     old_buffer = allocate(size_buffer);
+
+    if( size_buffer > 1 )
     while(fgets(old_buffer, size_buffer, fp)!=NULL )
     {
+	if( strlen(old_buffer) > 8000 )
+	  continue;
+    
 	/* DateStamp, 1-5 in the xferlog on the first line that has content */
-	if( ! once && strlen(old_buffer) < 8000 && strlen(old_buffer) > 5 )
+	if( ! once && strlen(old_buffer) > 5 )
 	{
 	    for(x=0; old_buffer[x]!='\n'; x++)
 	    {
@@ -303,51 +326,49 @@ main(int argc, char *argv[])
 
 		if(old_buffer[x]==' ' && old_buffer[x+1]!=' ')
 		z++;
-		if( z==5 )
+		if( z == 5 )
 		{
 		    /* Security bug fixed thanks to Tavis Ormandy */
 		    snprintf(datestamp, x+1, "%s", old_buffer);
-		    once=1;
+		    once = 1;
 		    break;
 		}
 	    }
 	}
+
 	strcpy(user, "");
-	file_size=0;
+	file_size = 0;
 
-	/* We skip these long entries for now ... FIXME */
-	if( strlen(old_buffer) < 8000 )
+	/* User */
+	sscanf(old_buffer, "%*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %s", user);
+
+	/* Size */
+	strcpy(temp, "");
+    	sscanf(old_buffer, "%*s %*s %*s %*s %*s %*s %*s %s", temp);
+	if( chars_are_digits(temp) )
+    	  sscanf(old_buffer, "%*s %*s %*s %*s %*s %*s %*s %llu", &file_size);
+
+    	/* UL or DL */
+    	sscanf(old_buffer, "%*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %s", action);
+
+    	/* File */
+    	sscanf(old_buffer, "%*s %*s %*s %*s %*s %*s %*s %*s %s", file);
+
+	/* Go thru the array looking for this user and add its ul/dl stats */
+	row = 0;
+	while( row < max_rows )
 	{
-	    /* User */
-	    sscanf(old_buffer, "%*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %s", user);
-	    
-	    /* Size */
-	    strcpy(temp, "");
-    	    sscanf(old_buffer, "%*s %*s %*s %*s %*s %*s %*s %s", temp);
-	    if( is_digitz(temp) )
+	    if( ! strcmp(userlist[row].user, user) )
 	    {
-    		sscanf(old_buffer, "%*s %*s %*s %*s %*s %*s %*s %llu", &file_size);
-	    }
-    	    /* UL or DL */
-    	    sscanf(old_buffer, "%*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %s", action);
-    	    /* File */
-    	    sscanf(old_buffer, "%*s %*s %*s %*s %*s %*s %*s %*s %s", file);
-	    /* Go thru the array looking for this user and add its ul/dl stats */
-	    row=0;
-	    while( row < max_rows )
-	    {
-		if( strstr(userlist[row].user, user) && strlen(userlist[row].user) == strlen(user) )
-		{
-    		    /* Upload */
-    		    if( strstr(action, "i") )
-    		      userlist[row].ul=userlist[row].ul+file_size/1024;
+    	        /* Upload */
+    	        if( strstr(action, "i") )
+    	          userlist[row].ul=userlist[row].ul+file_size/1024;
 
-		    /* Download */
-		    if( strstr(action, "o") )
-    		      userlist[row].dl=userlist[row].dl+file_size/1024;
-		}
-		row++;
+    		/* Download */
+		if( strstr(action, "o") )
+    		  userlist[row].dl=userlist[row].dl+file_size/1024;
 	    }
+	    row++;
 	}
     }
     fclose(fp);
@@ -357,8 +378,10 @@ main(int argc, char *argv[])
     free(dir);
     free(action);
     free(file);
+
     welcome = allocate(8192);
     welcome1 = allocate(8192);
+
     strcpy(welcome, "Welcome to ");
     strcat(welcome, server_name);
     strcat(welcome, " \%U\n");
@@ -422,54 +445,53 @@ main(int argc, char *argv[])
     while( 1 )
     {
         if( row > max_rows-1 )
-        {
-	    row=0;
-	}
+	  row = 0;
 
 	if( count >= max_rows * max_rows )
-	{
-	    break;
-	}
+	  break;
 
-	old_row=0;
-	timez=0;
-	timex=0;
+	old_row = 0;
+	timez = 0;
+	timex = 0;
+	
 	while( 1 )
 	{
 	    /* Greater or equal UL then any user in the list thats not flagged */
 	    if( userlist[row].ul >= userlist[old_row].ul  && ! userlist[row].ul_flag && ! userlist[old_row].ul_flag )
 	    {
-	        top_ul=row;
+	        top_ul = row;
 	        timez++;
 	    }
 	    /* Greater or equal DL then any user in the list thats not flagged */
 	    if( userlist[row].dl >= userlist[old_row].dl  && ! userlist[row].dl_flag && ! userlist[old_row].dl_flag )
 	    {
-	        top_dl=row;
+	        top_dl = row;
 		timex++;
 	    }
 
 	    /* .. Or if the user is flagged */
 	    if( userlist[old_row].ul_flag )
-	    timez++;
+	      timez++;
 
 	    if( userlist[old_row].dl_flag )
-	    timex++;
+	      timex++;
 	    
 	    /* TOP UPLOAD */
 	    if( timez > max_rows-1 && userlist[top_ul].ul !=0 && userlist[top_ul].ul_flag !=1 )
 	    {
 		z++;
-		if( z<=10 )
+		if( z <= 10 )
 		{
-	    	    userlist[top_ul].ul_flag=1;
+	    	    userlist[top_ul].ul_flag = 1;
 		    strcat(welcome, userlist[top_ul].user);
 		    strcat(welcome, ": ");
+
 		    sprintf(convert, "%llu", userlist[top_ul].ul);
 		    strcat(welcome, convert);
 		    strcat(welcome, " KB\n");
+		    
 		    /* Generate html */
-		    if( strlen(html)>1 )
+		    if( strlen(html) > 1 )
 		    {
 			strcat(html_welcome, "<tr><td class=left width=300>");
 			strcat(html_welcome, userlist[top_ul].user);
@@ -489,12 +511,13 @@ main(int argc, char *argv[])
 		x++;
 		if( x<=10 )
 		{
-	    	    userlist[top_dl].dl_flag=1;
+	    	    userlist[top_dl].dl_flag = 1;
 		    strcat(welcome1, userlist[top_dl].user);
 		    strcat(welcome1, ": ");
 		    sprintf(convert, "%llu", userlist[top_dl].dl);
 		    strcat(welcome1, convert);
 		    strcat(welcome1, " KB\n");
+		    
 		    /* Generate html */
 		    if( strlen(html) > 1 )
 		    {
@@ -543,8 +566,10 @@ main(int argc, char *argv[])
 	}
     }
     pclose(fp);
+
     
-    printf("%s\n", welcome);
+    printf("\n%s\n", welcome);
+
 
     /* Add welcome messages for all ftp users in proftpd.conf that should be updated */
     if( strlen(welcome_name) > 1 )
