@@ -1,6 +1,6 @@
 /*
  * GProFTPD - A GTK+ frontend for the ProFTPD standalone server.
- * Copyright (C) 2001, 2002, 2003, 2004, 2005 Magnus-swe <magnus-swe@telia.com>
+ * Copyright (C) 2001 - 2006 Magnus Loef (Magnus-swe) <magnus-swe@telia.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,32 +18,34 @@
  *
  */
 
+
 #include "../config.h"
 #include <gtk/gtk.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
-#include "allocate.h"
-#include "import_functions.h"
-#include "import_window.h"
-#include "dialogs.h"
 #include "support.h"
+#include "allocate.h"
+#include "widgets.h"
 #include "functions.h"
-#include "callbacks.h"
+#include "show_info.h"
+#include "import_functions.h"
+#include "system_defines.h"
+#include "populate_users.h"
+#include "populate_conf_tab.h"
+#include "reread_conf.h"
 
+extern char global_server_address[1024];
+extern char global_server_port[1024];
+extern char global_server_type[1024];
+extern char global_user_name[1024];
 
-/* Globals from callbacks.c */
-extern GtkWidget *userlist_treeview;
-extern char selected_server_address[8192];
-extern char selected_server_port[128];
-extern char selected_server_type[128];
-extern char info_buffer[4096];
-extern int online;
+extern int activated;
 
-#define NOBODY "nobody"
 #define GP_OSX_PASS_DUMP "/etc/gp_osx_passdump"
 
+
 /* Set at import_button_pressed */ 
 char import_root_dir[8192]="";
 char import_upload_dir[8192]="";
@@ -53,9 +55,8 @@ long num_imported=0;
 
 
 
-void
-add_import_users(GtkTreeModel *model, GtkTreePath *path,
-		 GtkTreeIter *iter, gpointer *user_data)
+void add_import_users(GtkTreeModel *model, GtkTreePath *path,
+		         GtkTreeIter *iter, struct w *widgets)
 {
     FILE *fp;
     long size_buffer;
@@ -64,6 +65,7 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
     char fixdir[4096]="";
     int limit_access=0, found_server=0;
     gchar *username;
+    gchar *info;
 
     gtk_tree_model_get(model, iter, 0, &username, -1);
 
@@ -76,8 +78,8 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
     	return;
     }
 
-    if( username[0]=='0'||username[0]=='1'||username[0]=='2'||username[0]=='3'||username[0]=='4' 
-    ||username[0]=='5'||username[0]=='6'||username[0]=='7'||username[0]=='8'||username[0]=='9') 
+    if( username[0]=='0' || username[0]=='1' || username[0]=='2' || username[0]=='3' || username[0]=='4' 
+    ||  username[0]=='5' || username[0]=='6' || username[0]=='7' || username[0]=='8' || username[0]=='9') 
     {
 	printf("Usernames cant have a digit first\n");
     	return;
@@ -104,25 +106,22 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
 
 
     address_buffer = allocate(8192+15);
-
     port_buffer = allocate(8192+3);               
 
-    if( strstr((char *)selected_server_type, "Virtualhost") )
+    if( strstr((char *)global_server_type, "Virtualhost") )
     {
-       sprintf(address_buffer, "<VirtualHost %s>\n", selected_server_address);
+       sprintf(address_buffer, "<VirtualHost %s>\n", global_server_address);
     }
     else
-       sprintf(address_buffer, selected_server_address);
+       sprintf(address_buffer, global_server_address);
      
-    sprintf(port_buffer, "Port %s\n", selected_server_port);
+    sprintf(port_buffer, "Port %s\n", global_server_port);
 
 
 
     /* Checks if the user exists in proftpd.conf if so just show a popup */
-    if((fp=fopen(GP_CONF,"r"))==NULL)
+    if((fp=fopen(PROFTPD_CONF,"r"))==NULL)
     {
-    	sprintf(info_buffer, "Error reading conf: %s\n", GP_CONF);
-	printf("Error: %s\n", info_buffer);
 	free(address_buffer);
 	free(port_buffer);
     	return;
@@ -140,16 +139,14 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
 
 
     /* Standard server selected, start checking users directly */
-    if( ! strstr((char *)selected_server_type, "Virtualhost") )
+    if( ! strstr((char *)global_server_type, "Virtualhost") )
       found_server=1;
 
-
+    if( size_buffer > 1 )
     while(fgets(old_buffer, size_buffer, fp)!=NULL)
     {
 	if( strlen(old_buffer) > 8000 )
 	{
-	    sprintf(info_buffer, "A line with over 8000 chars is not valid in: %s\n", GP_CONF);
-	    printf("Error: %s\n", info_buffer);
 	    fclose(fp);
 	    free(old_buffer);
     	    free(address_buffer);
@@ -158,18 +155,15 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
 	}
 
 
-        if( strstr("Virtualhost", (char *)selected_server_type) && ! found_server
+        if( strstr("Virtualhost", (char *)global_server_type) && ! found_server
 	&&  ! strcmp(old_buffer, address_buffer) ) 
 	{
 	    /* Lets see if its the same port as the selected one */
-	    /* If this server has the same port its the correct server .. the end. */
-
-	    while(fgets((char*)old_buffer, size_buffer, fp)!=NULL)
+	    /* If this server has the same port its the correct server */
+	    while(fgets(old_buffer, size_buffer, fp)!=NULL)
 	    {
 		if( strlen(old_buffer) > 8000 )
 		{
-		    sprintf(info_buffer, "A line with over 8000 chars is not valid in: %s\n", GP_CONF);
-		    printf("Error: %s\n", info_buffer);
 		    fclose(fp);
 		    free(old_buffer);
 	    	    free(address_buffer);
@@ -181,18 +175,15 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
 		 * else itll miss some vaules .. */
 		if( strstr(old_buffer, "Port") && ! strcmp(old_buffer, port_buffer) )
 		{
-		    found_server=1;
+		    found_server = 1;
 		    break;
 		}
-
 		
 		if( strstr(old_buffer, "</VirtualHost>") )
 		  break;
 	    }    
 	}
 
-
-	
 	/* Continue until we find the selected server */
 	if( ! found_server )
 	  continue;
@@ -207,14 +198,12 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
     }
 
 
-    /* Check user exist in this server.... */
-
-    while(fgets((char*)old_buffer, size_buffer, fp)!=NULL)
+    /* Check user exists in this server.... */
+    if( size_buffer > 1 )
+    while(fgets(old_buffer, size_buffer, fp)!=NULL)
     {
 	if( strlen(old_buffer) > 8000 )
 	{
-	    sprintf(info_buffer, "A line with over 8000 chars is not valid in: %s\n", GP_CONF);
-	    printf("Error: %s\n", info_buffer);
 	    fclose(fp);
 	    free(old_buffer);
 	    free(address_buffer);
@@ -226,9 +215,9 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
 	/* Does this user exist in this server... */
 	if( ! strcmp(old_buffer, user_check) && ! strstr(old_buffer, "AllowUser") )
 	{
-	    strcpy(info_buffer, "That user already exists in this server in proftpd.conf.\n");
-	    strcat(info_buffer, "Nothing was changed.\n");
-	    printf("Error: %s\n", info_buffer);
+	    info = g_strdup_printf(_("The user: \"%s\" was not imported\nbecause it already existed in this server.\n"), username);
+	    show_info(info);
+	    g_free(info);
 	    free(user_check);
 	    free(old_buffer);
 	    free(address_buffer);
@@ -238,7 +227,7 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
 	}
 
 	/* Dont match users in vhosts for the defualt server */
-	if( ! strstr((char *)selected_server_type, "Virtualhost") )
+	if( ! strstr((char *)global_server_type, "Virtualhost") )
 	{
 	    if( strstr(old_buffer, "<VirtualHost") )
 	      break;
@@ -276,7 +265,8 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
 	 strcat(fixdir, username);
       }
 
-    make_directory(fixdir, "0755");
+
+    make_dir_chmod((gchar *)fixdir, "0755");
 
 
     /* Make the usersettings with the specified directories */
@@ -329,7 +319,7 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
 	if( strlen(incoming_dir) > 1 )
 	  strcat(fixdir, incoming_dir);
 
-	make_directory(fixdir, "0777");
+	make_dir_chmod((gchar *)fixdir, "0777");
 
 	strcat(user_settings, "<Directory ");
 	strcat(user_settings, root_dir);
@@ -342,7 +332,7 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
 	if( strlen(incoming_dir) > 1 )
 	  strcat(user_settings, incoming_dir);
 	  
-	strcat(user_settings, "/*>\n");
+	strcat(user_settings, ">\n");
 	strcat(user_settings, "AllowOverwrite on\n");
 	strcat(user_settings, "<Limit UPLOAD_DIR_ALLOW LIST NLST MDTM SIZE SITE STAT APPE RETR STOR STOU MKD XMKD CWD XCWD PWD XPWD CDUP XCUP>\n");
 	strcat(user_settings, " AllowAll\n");
@@ -361,24 +351,20 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
       free(incoming_dir);
 
 
-    /* Add the new user_settings to the conf and to the right server 
+    /* Add the new user settings to the selected user and server 
      * and also add allow user to this server
      */
     
-    found_server=0;
+    found_server = 0;
 
     /* Standard server selected, start adding users directly */
-    if( ! strstr((char *)selected_server_type, "Virtualhost") )
-      found_server=1;
+    if( ! strstr((char *)global_server_type, "Virtualhost") )
+      found_server = 1;
 
 
     /* Add AllowUser UserName to proftpd.conf to the right server */
-    if((fp=fopen(GP_CONF,"r"))==NULL)
+    if((fp=fopen(PROFTPD_CONF,"r"))==NULL)
     {
-	strcpy(info_buffer, "Could not read AllowUser here:\n");
-    	strcat(info_buffer, GP_CONF);
-    	strcat(info_buffer, "\nRun gproftpd as root\n");
-	printf("Error: %s\n", info_buffer);
 	free(address_buffer);
 	free(port_buffer);
 	free(user_settings);
@@ -395,12 +381,11 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
     new_buffer = allocate(size_buffer+8192);
 
     /* Add the new user to the new conf */
+    if( size_buffer > 1 )
     while(fgets(old_buffer, size_buffer, fp)!=NULL)
     {
-	if( strlen(old_buffer) > 8000 )
+	if( strlen(old_buffer) > 4000 )
 	{
-	    sprintf(info_buffer, "A line with over 8000 chars is not valid in: %s\n", GP_CONF);
-	    printf("Error: %s\n", info_buffer);
 	    fclose(fp);
 	    free(old_buffer);
     	    free(address_buffer);
@@ -410,29 +395,17 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
 	    return;
 	}
 
-
 	strcat(new_buffer, old_buffer);
 
-
-        if( strstr("Virtualhost", (char *)selected_server_type) && ! found_server
+        if( strstr("Virtualhost", (char *)global_server_type) && ! found_server
 	&&  ! strcmp(old_buffer, address_buffer) )
 	{
 	    /* Lets see if its the same port as the selected one */
-	    /* If this server has the same port its the correct server .. the end. */
-	    while(fgets((char*)old_buffer, size_buffer, fp)!=NULL)
+	    /* If this server has the same port its the correct server. */
+	    while(fgets(old_buffer, size_buffer, fp)!=NULL)
 	    {
-		if( strlen(old_buffer) > 8000 )
-		{
-		    sprintf(info_buffer, "A line with over 8000 chars is not valid in: %s\n", GP_CONF);
-		    printf("Error: %s\n", info_buffer);
-		    fclose(fp);
-		    free(old_buffer);
-		    free(new_buffer);
-	    	    free(address_buffer);
-		    free(port_buffer);
-		    free(user_settings);
-		    return;
-		}
+		if( strlen(old_buffer) > 4000 )
+		  continue;
 
 		strcat(new_buffer, old_buffer);
 
@@ -441,7 +414,7 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
 		 * else itll miss some vaules .. */
 		if( strstr(old_buffer, "Port") && ! strcmp(old_buffer, port_buffer) )
 		{
-		    found_server=1;
+		    found_server = 1;
 		    break;
 		}
 		
@@ -463,16 +436,16 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
 	    strcat(new_buffer, "  AllowUser ");
 	    strcat(new_buffer, username);
 	    strcat(new_buffer, "\n");
-	    limit_access=1; /* just incase so we just change the first occurance */
+	    limit_access = 1; /* just incase so we just change the first occurance */
 	    
 	    /* Add the user after </Limit> */
-	    while(fgets((char*)old_buffer, size_buffer, fp)!=NULL)
+	    while(fgets(old_buffer, size_buffer, fp)!=NULL)
 	    {
 	       strcat(new_buffer, old_buffer);
 	       if( strstr(old_buffer, "</Limit>") && limit_access==1 )
 	       {
 		    /* Only add it once */
-		    limit_access=2;
+		    limit_access = 2;
 		    /* The user was added after </Limit> */		    
 
 		    /* Global */
@@ -487,7 +460,7 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
 	if( strstr(old_buffer, "</Anonymous") && limit_access==1 )
 	{
 	   /* Only add it once */
-	   limit_access=2;
+	   limit_access = 2;
 	   /* The user was added after the first user */
 
 	    /* Global */
@@ -504,12 +477,8 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
 
 
     /* Write the new configuration */
-    if((fp=fopen(GP_CONF,"w+"))==NULL)
+    if((fp=fopen(PROFTPD_CONF,"w+"))==NULL)
     {
-	strcpy(info_buffer, "Could not write the new userinformation to:\n");
-    	strcat(info_buffer, GP_CONF);
-    	strcat(info_buffer, "\nRun gproftpd as root\n");
-	printf("Error: %s\n", info_buffer);
 	free(new_buffer);
         return;
     }
@@ -523,128 +492,25 @@ add_import_users(GtkTreeModel *model, GtkTreePath *path,
 }
 
 
-void
-on_import_window_map                   (GtkWidget       *widget,
-                                        gpointer         user_data)
+
+
+void import_button_clicked(GtkButton *button, struct w *widgets)
 {
-    /* List all users in passwd but not root or the server-user */
-    FILE *fp;
-    char *old_buffer, *new_buffer;
-    GtkWidget *import_treeview;
-    GtkTreeViewColumn *import_users_column;
-    GtkListStore *import_users_model;
-    GtkTreeIter iter;
+    /* Add all the marked users to the selected server, 
+     * make dirs and allow them to login */
     GtkTreeSelection *selection;
-    GtkCellRenderer *cell;
-    long size_buffer;
-    int i;
-    gchar *utf8=NULL;            
-
-    import_treeview = lookup_widget(GTK_WIDGET(widget), "import_treeview");
-
-    /* Setup the server list */
-    import_users_model = gtk_list_store_new(1, G_TYPE_STRING);
-    gtk_tree_view_set_model(GTK_TREE_VIEW(import_treeview), GTK_TREE_MODEL(import_users_model));
-    cell = gtk_cell_renderer_text_new();
-    import_users_column = gtk_tree_view_column_new_with_attributes("Users", cell, "text", 0, NULL);
-    gtk_list_store_clear(import_users_model);
-    gtk_tree_view_append_column(GTK_TREE_VIEW(import_treeview), GTK_TREE_VIEW_COLUMN(import_users_column));
-    
-    /* Selection is multiple */
-    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(import_treeview));
-
-    gtk_tree_selection_set_mode(GTK_TREE_SELECTION(selection), GTK_SELECTION_MULTIPLE);
-
-    if((fp=fopen(GP_PASSWD, "r"))==NULL)
-    {
-	/* Report nothing */
-    }
-    else
-      {
-    	 fseek(fp, 0, SEEK_END);
-         size_buffer = ftell(fp);
-	 rewind(fp);
-
-         old_buffer = allocate(size_buffer);          
-	 if( old_buffer==NULL )
-	 {
-	    printf("Cant allocate enough memory for user import\n");
-	    fclose(fp);
-	    return;
-	 }
-
-         new_buffer = allocate(8192);          
-	 if( new_buffer==NULL )
-	 {
-	    printf("Cant allocate enough memory for user import\n");
-	    fclose(fp);
-	    free(old_buffer);
-	    return;
-	 }
-
-         while(fgets(old_buffer, size_buffer, fp)!=NULL) 
-         {
-
-	    if( strlen(old_buffer) > 10 && strlen(old_buffer) < 8000 )
-	    {
-		for(i=0; old_buffer[i]!='\0'; i++)
-		  if( old_buffer[i]==':' )
-		    break;
-
-		strcpy(new_buffer, old_buffer);
-		new_buffer[i]='\0';
-		
-		/* Dont insert root or the user the server runs as */
-		if( !strcmp(new_buffer, "root") || !strcmp(new_buffer, NOBODY) )
-		  continue;
-		
-		gtk_list_store_append(GTK_LIST_STORE(import_users_model), &iter);
-	    	utf8 = g_locale_to_utf8(new_buffer, strlen(new_buffer), NULL, NULL, NULL);
-	    	gtk_list_store_set(GTK_LIST_STORE(import_users_model), &iter, 0, utf8, -1);
-	    }
-
-	 }
-         fclose(fp);
-         free(old_buffer);
-         free(new_buffer);
-      }
-
-      if( utf8 != NULL )
-        g_free(utf8);
-}
-
-
-void
-on_import_button_clicked               (GtkButton       *button,
-                                        gpointer         user_data)
-{
-    /* add all the marked users to the conf make dirs and allow them to login */
-    GtkWidget *import_window;
-    GtkWidget *import_treeview;
-    GtkWidget *info_window;
-    GtkTreeSelection *selection;
-    GtkWidget *import_home_dir_entry;
-    GtkWidget *import_upload_entry;
-    GtkWidget *import_namebased_checkbutton;
-    GtkWidget *import_upload_checkbutton;
     G_CONST_RETURN gchar *g_home_dir;
     G_CONST_RETURN gchar *g_upload_dir;
-    num_imported=0;
-    
-    import_window = lookup_widget(GTK_WIDGET(button), "import_window");
-    import_treeview = lookup_widget(GTK_WIDGET(button), "import_treeview");
-    import_home_dir_entry = lookup_widget(GTK_WIDGET(button), "import_home_dir_entry");
-    import_upload_entry = lookup_widget(GTK_WIDGET(button), "import_upload_entry");
-    import_namebased_checkbutton = lookup_widget(GTK_WIDGET(button), "import_namebased_checkbutton");
-    import_upload_checkbutton = lookup_widget(GTK_WIDGET(button), "import_upload_checkbutton");
+    gchar *info=NULL;
 
     /* Globals */
+    num_imported = 0;
     strcpy(import_root_dir, "");
     strcpy(import_upload_dir, "");
 
 
-    g_home_dir = gtk_entry_get_text(GTK_ENTRY(import_home_dir_entry));
-    if( strlen(g_home_dir) <= 8000 )
+    g_home_dir = gtk_entry_get_text(GTK_ENTRY(widgets->import_home_entry));
+    if( strlen(g_home_dir) <= 4000 )
     {
         sprintf(import_root_dir, g_home_dir);
     }
@@ -654,8 +520,8 @@ on_import_button_clicked               (GtkButton       *button,
          return;
       }
 
-    g_upload_dir = gtk_entry_get_text(GTK_ENTRY(import_upload_entry));
-    if( strlen(g_upload_dir) <= 8000 )
+    g_upload_dir = gtk_entry_get_text(GTK_ENTRY(widgets->import_upload_entry));
+    if( strlen(g_upload_dir) <= 4000 )
     {
         sprintf(import_upload_dir, g_upload_dir);
     }
@@ -665,60 +531,46 @@ on_import_button_clicked               (GtkButton       *button,
          return;
       }
 
-    if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(import_upload_checkbutton)) )
-    {
-        printf("FIXME: Add upload directory (import_functions.c)\n");
-
-    }
-
 
     /* Call foreach on the users */
-    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(import_treeview));
+    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widgets->import_treeview));
 
     gtk_tree_selection_selected_foreach(GTK_TREE_SELECTION(selection),
-                                       (GtkTreeSelectionForeachFunc) &add_import_users,
-				       (gpointer) user_data);
+                                       (GtkTreeSelectionForeachFunc) &add_import_users, widgets);
 
 
-    /* Update the userlist and the entire tab. */
-    list_users(userlist_treeview, NULL);
-    list_entire(userlist_treeview, NULL);
+    /* Update the user treeview and settings and the conf tab. */
+    populate_users(widgets);
+    populate_conf_tab(widgets);
 
-    gtk_widget_destroy(import_window);
+    gtk_widget_destroy(widgets->import_window);
 
-    sprintf(info_buffer, _("Total number of imported users: %ld\n"), num_imported);
-    info_window = create_info_window();
-    gtk_widget_show(info_window);
+    info = g_strdup_printf(_("Total number of imported users: %ld\n"), num_imported);
+    show_info(info);
 
-    /* ReRead */
-    if( online )
-      system("killall -1 proftpd"); 
+    reread_conf(widgets);
 }
 
 
-void
-on_import_namebased_checkbutton_toggled (GtkToggleButton *togglebutton,
-                                         gpointer         user_data)
+void import_namebased_checkbutton_toggled(GtkToggleButton *togglebutton, struct w *widgets)
 {
     if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(togglebutton)) )
     {
 	/* Globals */
-	import_to_usernamed_dirs=1;
+	import_to_usernamed_dirs = 1;
     }
     else
-      import_to_usernamed_dirs=0;
+      import_to_usernamed_dirs = 0;
 }
 
 
-void
-on_import_upload_checkbutton_toggled (GtkToggleButton *togglebutton,
-                                      gpointer         user_data)
+void import_upload_checkbutton_toggled(GtkToggleButton *togglebutton, struct w *widgets)
 {
     if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(togglebutton)) )
     {
 	/* Global */
-	import_add_upload_dir=1;
+	import_add_upload_dir = 1;
     }
     else
-      import_add_upload_dir=0;
+      import_add_upload_dir = 0;
 }
